package sample.job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;
import org.springframework.scheduling.quartz.QuartzJobBean;

import com.poscoict.glueframework.biz.control.GlueBizControlConstants;
import com.poscoict.glueframework.biz.control.GlueBizController;
import com.poscoict.glueframework.biz.control.GlueBizProvider;
import com.poscoict.glueframework.context.GlueContext;
import com.poscoict.glueframework.context.GlueDefaultContext;
import com.poscoict.glueframework.context.GlueStaticContext;
import com.poscoict.glueframework.dao.vo.GlueAuditAttributes;

public class GlueQuartzJobBean extends QuartzJobBean
{
    protected Logger logger = LoggerFactory.getLogger( getClass() );

    @Override
    protected final void executeInternal( JobExecutionContext context ) throws JobExecutionException
    {
        try
        {
            this.logger.trace( "##### {}", context );
//            if ( this.logger.isTraceEnabled() )
//            {
//                this.logger.trace( "{} is starting by {}.", context.getJobDetail().getName(), context.getTrigger().getName() );
//                this.logger.trace( "{}", context.getJobDetail().getJobDataMap() );
//            }

            this.beforeExecuteJob( context );
            this.logger.trace( "{}", context );
            this.logger.trace( "TriggerKey : {}", context.getTrigger().getKey() );
            this.logger.trace( "Trigger : {} {}", context.getTrigger().getMisfireInstruction(), context.getTrigger().getPriority() );
            this.logger.trace( "JobKey : {}", context.getJobDetail().getKey() );
            this.logger.trace( "JobDetail : {} {} {} {}", context.getJobDetail().isConcurrentExectionDisallowed(), context.getJobDetail().isDurable(), context.getJobDetail().isPersistJobDataAfterExecution(), context.getJobDetail().requestsRecovery() );
            this.logger.trace( "JobDataMap : {}", context.getJobDetail().getJobDataMap() );
            this.logger.trace( "{}", context.getMergedJobDataMap() );
//            this.executeJob( context );
            this.afterExecuteJob( context );
        } finally
        {
            if(context==null){
                this.logger.info( "Job Elasped time... [{}]ms", "unknown" );
            }else{
                this.logger.info( "Job Elasped time... [{}]ms", context.getJobRunTime() );
            }
        }
    }

    /**
     * @param context
     */
    protected void executeJob( JobExecutionContext context )
    {
        GlueContext ctx = new GlueDefaultContext();
        ctx.putAll( context.getMergedJobDataMap() );

        // Audit Start //
        if ( GlueStaticContext.isAudit() )
        {
            ctx.setAuditAttributes( new GlueAuditAttributes( ctx ) );
        }
        // Audit End //

        MDC.put( GlueBizControlConstants.SERVICE_NAME, (String) ctx.get( GlueBizControlConstants.SERVICE_NAME ) );
        GlueBizController bizIF = GlueBizProvider.getController();
        bizIF.doAction( ctx );
        MDC.remove( GlueBizControlConstants.SERVICE_NAME );
    }

    /**
     * @param context
     */
    protected void beforeExecuteJob( JobExecutionContext context )
    {}

    /**
     * @param context
     */
    protected void afterExecuteJob( JobExecutionContext context )
    {}}
